@@ -58,6 +58,8 @@ def docs():
         "[31] ssap.colorlist()\n"
         "[32] ssap.colormap()\n"
         "[33] ssap.scatter05box(df, parameter_x, parameter_y, title)\n"
+        "[34] ssap.scatter06histlr(df, parameter_x, parameter_y, bins, title)\n"
+        "[35] ssap.scatter07boxlr(df, parameter_x, parameter_y, title)\n"
     )
     # explain details:
     # print(
@@ -1305,10 +1307,6 @@ def scatter05box(df, parameter_x, parameter_y, title):
     plt.show()
 
 
-
-
-
-# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> [to be validated. update:12/28]
 # scatter plot + hist plot + linear regression
 def scatter06histlr(df, parameter_x, parameter_y, bins, title):
     import matplotlib.pyplot as plt
@@ -1457,6 +1455,124 @@ def scatter07boxlr(df, parameter_x, parameter_y, title):
     plt.show()
 
 
+# scatter plot + linear regression + 95%ci error zone.
+def scatter08lrci(df, parameter_x, parameter_y, title):
+    import numpy as np
+    import matplotlib.pyplot as plt
+    from scipy import stats
+    from scipy.stats import linregress
+
+    df = df
+    x = df[parameter_x].tolist()
+    y = df[parameter_y].tolist()
+    coef = np.polyfit(x, y, 1)
+    poly1d_fn = np.poly1d(coef)
+
+    lr = linregress(x, y)
+    lr_label = '[lr] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
+        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
+
+    sample = y - poly1d_fn(x)
+
+    #     print(x)
+
+    def mean_ci(level, sample):
+        confidence_level = level
+        degrees_freedom = sample.size - 1
+        # df:degree of freedom, df = N-k (N-sample number, k-restricted condition/variables).
+        # df=N-1 (restricted condition, mean=80). more df → more evidence → more convincible conslusion.
+        sample_mean = np.mean(sample)
+        sample_mean_standard_error = stats.sem(sample)
+        confidence_interval = stats.t.interval(confidence_level, degrees_freedom, sample_mean,
+                                               sample_mean_standard_error)
+        # got small number sample case, use t-test.
+        return sample_mean, confidence_interval[0], confidence_interval[1]
+
+    sample_mean, lower, upper = mean_ci(level=0.95, sample=sample)
+
+    x1 = list(set(x))
+    x1.sort()
+
+    est_upper = poly1d_fn(x1) + sample_mean + upper
+    est_lower = poly1d_fn(x1) + sample_mean + lower
+    #     print(x1)
+    #     print(est_upper)
+
+    fig, ax = plt.subplots()
+
+    scatter = ax.scatter(x=x, y=y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='orange')
+    plt.plot(x, poly1d_fn(x), '-', color='blue', lw=0.5, label=lr_label)
+    #     plt.scatter(x, est_upper, c = 'b')
+    #     plt.scatter(x, est_lower, c = 'b')
+    plt.fill_between(x1, est_upper, est_lower, color='lightblue', alpha=0.5, label='[ci] 95% zone')
+
+    plt.grid(linestyle='-', lw=0.5)
+
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+    plt.legend(loc='upper left')
+
+    plt.title(title)
+    plt.show()
+
+
+# scatter plot + linear regression + ±n*sigma zone.
+def scatter09lr6sigma(df, parameter_x, parameter_y, sigma, title):
+    import numpy as np
+    import matplotlib.pyplot as plt
+    from scipy import stats
+    from scipy.stats import linregress
+
+    df = df
+    x = df[parameter_x].tolist()
+    y = df[parameter_y].tolist()
+    coef = np.polyfit(x, y, 1)
+    poly1d_fn = np.poly1d(coef)
+
+    lr = linregress(x, y)
+    lr_label = '[lr] y = ' + str("{0:.3g}".format(lr[0])) + ' *x + ' + str("{0:.3g}".format(lr[1])) + ' (r=' + str(
+        "{0:.3g}".format(lr[2])) + '|p=' + str("{0:.3g}".format(lr[3])) + '|σ=' + str("{0:.3g}".format(lr[4])) + ')'
+
+    sample = y - poly1d_fn(x)
+    sample_mean = np.mean(sample)
+    sample_stderr = np.std(sample)
+
+    x1 = list(set(x))
+    x1.sort()
+    n = sigma
+
+    est_upper = poly1d_fn(x1) + sample_mean + n * sample_stderr
+    est_lower = poly1d_fn(x1) + sample_mean - n * sample_stderr
+
+    fig, ax = plt.subplots()
+
+    scatter = ax.scatter(x=x, y=y, edgecolor='black', lw=0.5, alpha=0.7, facecolor='orange')
+    plt.plot(x, poly1d_fn(x), '-', color='blue', lw=0.5, label=lr_label)
+
+    if n == 1:
+        cover = '65.26%'
+    elif n == 2:
+        cover = '95.44%'
+    elif n == 3:
+        cover = '99.74%'
+    else:
+        cover = '?, sigma should be 1,2 or 3.'
+
+    plt.fill_between(x1, est_upper, est_lower, color='lightblue', alpha=0.5,
+                     label='±' + str(n) + ' sigma zone (cover ' + cover + ')')
+
+    plt.grid(linestyle='-', lw=0.5)
+
+    plt.xlabel(parameter_x)
+    plt.ylabel(parameter_y)
+    plt.legend(loc='upper left')
+
+    plt.title(title)
+    plt.show()
+
+# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> [to be validated. update:12/28]
+
+
 
 
 # show correlation line + error zone. with scatter plot:
@@ -1471,4 +1587,6 @@ def scatter07boxlr(df, parameter_x, parameter_y, title):
 # SSAP用户报告里绘制的图
 # 多项式回归拟合
 # 聚类算法 DBSCAN.
-# regression model:
\ No newline at end of file
+# regression model:
+# 多元线性回归：https://cloud.tencent.com/developer/article/1059976
+# 移动平均模型（ARIMA）
\ No newline at end of file
